/*
	(C) 1995-96 AROS - The Amiga Research OS
	$Id: freeintuimessage.c,v 1.3 2010/02/28 14:26:17 jacadcaps Exp $
 
	Desc:
	Lang: english
*/
#include <proto/utility.h>
#include "intuition_intern.h"

/*****i***********************************************************************
 
	NAME */
#include <proto/intuition.h>

AROS_LH1(void, FreeIntuiMessage,

		 /*  SYNOPSIS */
		 AROS_LHA(struct IntuiMessage *, imsg, A0),

		 /*  LOCATION */
		 struct IntuitionBase *, IntuitionBase, 149, Intuition)

/*  FUNCTION
	Private to AROS: free an IntuiMessage previously allocated
	with AllocIntuiMessage.
 
	INPUTS
	imsg - The IntuiMessage. May be NULL.
 
	RESULT
	none
 
	NOTES
 
	EXAMPLE
 
	BUGS
 
	SEE ALSO
 
	INTERNALS
 
	HISTORY
 
*****************************************************************************/
{
	AROS_LIBFUNC_INIT
	AROS_LIBBASE_EXT_DECL(struct IntuitionBase *,IntuitionBase)

	DEBUG_FREEINTUIMESSAGE(dprintf("FreeIntuiMessage: Msg 0x%lx\n", imsg));

	ASSERT_VALID_PTR_OR_NULL(imsg);

#define EIM(x) ((struct ExtIntuiMessage *)(x))

	if (imsg)
	{
		ObtainSemaphore(&IBase->IDCMPCacheLock);

		if (EIM(imsg)->eim_TabletData)
		{
			if (EIM(imsg)->eim_TabletData->td_TagList) FreeTagItems(EIM(imsg)->eim_TabletData->td_TagList);
			FreePooled(IBase->IDCMPPool,EIM(imsg)->eim_TabletData,sizeof (struct TabletData));
			EIM(imsg)->eim_TabletData = NULL;
		}

		ADDHEAD(&IBase->IDCMPCacheList,imsg);

		ReleaseSemaphore(&IBase->IDCMPCacheLock);
	}

	DEBUG_FREEINTUIMESSAGE(dprintf("FreeIntuiMessage: done\n"));
	AROS_LIBFUNC_EXIT
}

